import os
import json
import numpy as np
import pandas as pd
import pickle as pk

def dump_json(obj, fpath, encoding='UTF-8', is_ascii=False, indent=4):
    with open(fpath, 'w', encoding=encoding) as f:
        json.dump(obj, f, ensure_ascii=is_ascii, indent=indent)

def load_json(fpath, encoding='UTF-8'):
    with open(fpath, 'r', encoding=encoding) as f:
        return json.load(f)

def dump_pkl(obj, fpath):
    with open(fpath, 'wb') as f:
        pk.dump(obj, f)

def load_pkl(fpath):
    with open(fpath, 'rb') as f:
        return pk.load(f)

def dump_npy(obj, fpath):
    np.save(fpath, obj)

def load_npy(fpath):
    return np.load(fpath)

def dump_csv(obj, fpath, header=False, index=False):
    pd.DataFrame(obj).to_csv(fpath, header=header, index=index)

def load_csv(fpath, header=None):
    pd.read_csv(fpath, header=header)

def load(fpath, **kwargs):
    load_dict = {
        '.json': load_json,
        '.pkl': load_pkl,
        '.npy': load_npy,
        '.csv': load_csv
    }
    _, ext = os.path.splitext(fpath)
    return load_dict[ext](fpath)

def dump(obj, fpath, **kwargs):
    dump_dict = {
        '.json': dump_json,
        '.pkl': dump_pkl,
        '.npy': dump_npy,
        '.csv': dump_csv
    }
    _, ext = os.path.splitext(fpath)
    return dump_dict[ext](obj, fpath, **kwargs)

def mkdir(dir_path):
    os.makedirs(dir_path, exist_ok=True)

if __name__ == "__main__":
    d = {'Name': 'Testing', 'Arr': [1, 2, 3], '測試': '項目'}
    dump(d, 'a.json', is_ascii=True)
    # dump_json(d, './a.json')
    # dd = load_json('./a.json')
    # assert d == dd

    # dump_pkl(d, './a.pkl')
    # dd = load_pkl('./a.pkl')
    # assert d == dd

    # dump(d, 'a.json')
    # dump(d, 'a.pkl')
    # dump(d, 'a.csv')
    # a = load('a.json')
    # b = load('a.pkl')
    # c = load('a.csv')
    # assert a == b

    # print('Success!')

    os.remove('a.json')
    # os.remove('a.pkl')
    # os.remove('a.csv')



import os
import numpy as np
import datetime as dt
import matplotlib.pyplot as plt

def to_categorical(arr):
    s = set(arr)
    d = dict(zip(list(s), range(len(s))))
    rtn = list(map(d.get, arr))

    return np.array(rtn), d

def plot_labels(data, labels, xlim=None, ylim=None, save=None):
    [plt.plot(data[lab], label=lab) for lab in labels]
    plt.legend()
    
    if xlim:
        plt.xlim(*xlim)
    
    if ylim:
        plt.ylim(*ylim)
    
    if save:
        plt.savefig(save)
        plt.clf()
    
    return plt

def walk_dir(target_dir, return_file_name=False, return_dir_name=False):
    for dir_path, _, file_list in os.walk(target_dir):
        for file_name in file_list:
            full_path = os.path.join(dir_path, file_name)
            rtn = [full_path]
            if return_file_name:
                rtn.append(file_name)
            if return_dir_name:
                rtn.append(dir_path)
            yield rtn

class TimeCost:
    def __init__(self, msg='Time Cost', verbose=True):
        self.ts = None
        self.verbose = verbose
        self.msg = msg
    
    def __enter__(self):
        self.ts = dt.datetime.now()
    
    def __exit__(self, *args):
        self.ts = dt.datetime.now() - self.ts
        if self.verbose:
            print(f'{self.msg}: {self.ts.total_seconds()}s')

if __name__ == "__main__":
    # print(to_categorical([1, 2, 3, 2, 1]))
    # print(to_categorical(['香草', '巧克力', '草莓', '巧克力', '香草']))
    # with TimeCost('Sleep Time'):
    #     import time
    #     time.sleep(1)
    for [fp] in walk_dir('./'):
        print(fp)



import setuptools

setuptools.setup(
    name="penut-utils",
    version="0.0.1",
    author="PenutChen",
    author_email="penut85420@gmail.com",
    description="This is a collection of my useful functions.",
    long_description=open('README.md', 'r', encoding='UTF-8').read(),
    long_description_content_type="text/markdown",
    url="https://github.com/penut85420/Penut",
    packages=setuptools.find_packages(),
    install_requires=[
        "numpy", "pandas", "matplotlib"
    ],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.6',
)
